<?php
/**
 * 
 * @package algorithmform
 */
class algorithmForm{
	private $contentSize;
	private $moveTo;
	private $js_dlg;
	private $dlg_form;
	private $boxes;
	private $upload_form;
	private $help_cont;
	private $param_desc;
	private $alg_title;
	private $msgDiv;

	/**
	 * Create a new dialog box for the form
	 * 
	 *
	 *@param string contains the Content Size values --> 'width,height'
	 *@param string the position of the dialog box	 --> 'x,y'
	 */
	public function __construct($contentSize,$moveTo,$div){
		$this->msgDiv = '';
		$this->contentSize = $contentSize;
		$this->moveTo = $moveTo;
		$this->param_desc = '';
		$this->js_dlg = "
		dialog = new Ext.LayoutDialog('$div', {
			modal:true,
			shadow:true,
			proxyDrag: true,
			collapsible: false,
			resizable: false,
			//closable: false,	
			center: { 
				autoScroll:true,
				tabPosition: 'top',
				closeOnTab: true,
				alwaysShowTabs: true
			}
		});
		//dialog.addKeyListener(27, dialog.hide, dialog);
		dialog.show();
		dialog.setContentSize($contentSize);
		dialog.moveTo($moveTo);

		var layout = dialog.getLayout();
		layout.beginUpdate();
		layout.add('center', new Ext.ContentPanel('cont_div', {title: 'Algorithm', closable: false}));
		layout.add('center', new Ext.ContentPanel('help_div', {title: 'Help', closable: false}));		
		layout.getRegion('center').showPanel('cont_div');
		layout.endUpdate();		
		";
		
	}
	/**
	 * Create a new combo box for the form
	 * 
	 *
	 *@param string contains the Id of the combo box it must be unique
	 *@param string the label for the combo box
	 *@param string the id and name of the options in the combo box --> "'id','column_name'"
	 *@param string the options who have the combo box  --> "['id','name'], ['id','name'] ..."
	 *@param string the text to show when the combo box is empty
	 *@param string the option who will be show  
	 */
	public function addCombo($id, $label, $fieldNames, $fields, $emptyText, $displayField, $description){
		$this->param_desc .= $description . '<br>';
		$store = $id ."_store";
		$rec = $id ."_rec";
		$fds = explode(",",$fieldNames);
		//$store.add(new $rec({ ".$fds[0] .": '__field1__', ".$fds[1].": 'field1 varchar(255)' }));
		$this->boxes .= "
		$rec = Ext.data.Record.create( [
			{name: ".$fds[0] .", type: 'string'},
			{name: ".$fds[1] .", type: 'string'}
		]);		
		
		$store = new Ext.data.SimpleStore({
					fields: [$fieldNames],
					data : [$fields]
				});
		
		alg_Form.add(
			new Ext.form.ComboBox({
				fieldLabel: '$label',				
				name : '$id',
				id: '$id',
				store: $store,
				displayField:'$displayField',
				typeAhead: true,				
				mode: 'local',
				triggerAction: 'all',
				emptyText:'$emptyText',
				selectOnFocus:true,
				width:180,
				allowBlank:false,
				editable:false
			})
		);
		";	
	}
	/**
	 * Create a new number field for the form
	 * 
	 *
	 *@param string contains the Id of the number field it must be unique
	 *@param string the label for the number field
	 *@param number default value for the number field
	 *@param string the regular expression to evaluate in this number field (posix) ex. "/^\d+$/"
	 *@param string the text to show if regular expression evaluation fails
	 *@param string true/false to allow blank
	 *@param string evaluate the value of the number field with the given string
	 *@param string the text to show if the value of the number field not match with the value given by $valueIn
	 */
	public function addNumberField($id, $label, $default, $regExp, $regexText, $allowBlank, $allowDec, $valueIn, $invalidText, $description){
		$this->param_desc .= $description . '<br>';
		$this->boxes .= "
		alg_Form.add(
			new Ext.form.NumberField({
				fieldLabel: '$label',
				name: '$id',
				id:'$id',
				width:175,
				allowBlank:$allowBlank,
				allowDecimals:$allowDec,
				regex: $regExp,
				regexText: '$regexText', 
				value: $default				
		";
		if($valueIn == ''){
			$this->boxes .= "
				})
			);
			";
			
		}else{
			$this->boxes .= ",
					invalidText: '$invalidText',
					validator: function(value){
						if(value.search(/$valueIn/) == -1){
							return false;
						}else{
							return true;
						}
					}
				})
			);
			";			
		}	
	}
	/**
	 * Create a new text field for the form
	 * 
	 *
	 *@param string contains the Id of the text field it must be unique
	 *@param string the label for the text field
	 *@param string the regular expression to evaluate in this text field (posix) ex. "/^\d+$/"
	 *@param string the text to show if regular expression evaluation fails
	 *@param string true/false to allow blank
	 *@param string evaluate the value of the text field with the given string
	 *@param string the text to show if the value of the text field not match with the value given by $valueIn
	 *@param string used for input files...
	 */
	public function addTextField($id, $label, $regExp, $regexText, $allowBlank, $valueIn, $invalidText, $beforeext, $afterext, $description,$length){
		$this->param_desc .= $description . '<br>';
		$this->boxes .= "
		alg_Form.add(
			new Ext.form.TextField({
				fieldLabel: '$label',
				name: '$id',
				id:'$id',
				width:175,
				allowBlank:$allowBlank,				
				regex: $regExp,
				regexText: '$regexText',
				maxLength: $length				
		";
		if($valueIn == ''){
			$this->boxes .= "
				})
			);
			";
			
		}else{
			$this->boxes .= ",
					invalidText: '$invalidText',
					validator: function(value){
						value = '$beforeext' + value + '$afterext';
						var txt = '$valueIn';
						txt = txt.split('&&&');
						var ban = 1;
						for(var i=0; i<txt.length; i++){
							if(value.toLowerCase() == txt[i]){							
								ban = 0;
								break;
							}else{
								ban = 1;
							}
						}
						if(ban == 0){
							return false;
						}else{
							return true;
						}
					}
				})
			);
			";			
		}	
	}	
	
	
	public function addEditor($id,$label,$width, $height){
		/*$this->boxes .= "
		alg_Form.container({labelAlign: 'top'},		
			new Ext.form.HtmlEditor({
				id:'$id',
				fieldLabel:'$label',
				width:$width,
				height:$height,
				enableFont: false,
				enableLinks: false,
				enableFontSize: false,
				enableFormat: false,
				enableColors:false,
				enableLists:false,
				enableSourceEdit: false
				
			})
		);
		";*/
		$this->boxes .= "
		alg_Form.container({labelAlign: 'top'},		
			new Ext.form.TextArea({
				id:'$id',
				fieldLabel:'$label',
				width:$width,
				height:$height				
			})
		);
		";

	}
/*public function addMultiselect(){
		$this->boxes.="	
		var itemSelector = new Ext.ux.ItemSelector({
			name: 'abc', 
			dataFields:['code', 'desc'], 
			fromData:[['1', 'One'], ['2', 'Two'], ['3', 'Three'], ['4', 'Four'], ['5', 'Five']],
			toData:[['6', 'Six']],
			msWidth: 100,
			msHeight: 200,
			valueField: 'code',
			displayField: 'desc',
			fieldLabel: 'Item Selector' name: 'abc', 
			dataFields:['code', 'desc'], 
			fromData:[['1', 'One'], ['2', 'Two'], ['3', 'Three'], ['4', 'Four'], ['5', 'Five']],
			toData:[['6', 'Six']],
			msWidth: 80,
			msHeight: 100,
			valueField: 'code',
			displayField: 'desc',
			fieldLabel: 'Item Selector'
		});
		alg_Form.container({labelAlign: 'top'},itemSelector);
		";
}*/
	function addMultiselect($name, $fromLabel, $toLabel, $dataFields, $fromData,$toData, $width, $height, $valueField, $displayField, $fieldLabel){
		$this->boxes.="
		var itemSelector = new Ext.ux.ItemSelector({
			name: '$name', 
			dataFields:$dataFields, 
			fromData:$fromData,
			toData:$toData,
			msWidth: $width,
			msHeight: $height,
			valueField: '$valueField',
			displayField: '$displayField',
			fieldLabel: '$fieldLabel',
			fromLabel: '$fromLabel',
			toLabel: '$toLabel'
		});
		alg_Form.container({labelAlign: 'top',hideLabels:true},
		itemSelector);
		";
	}
	public function addMsgDiv($id,$html){
		$this->msgDiv .="
		var msg = Ext.get('$id');
		msg.createChild({
			tag:'div',
			html:'<br>$html'
		});
		";
	}
	public function setContainer($id){
		$this->boxes .= "alg_Form.container({id: '$id'}); alg_Form.end();";
	}
	/**
	 * Sets a combo event between two comboboxes, this function only apply
	 * in Change Column Type filter
	 * 
	 *
	 *@param string contains the Id of the base combo_box
	 *@param string contains the Id of the destiny combo_box
	 *@param string th ids of the combo box "'id','column_name'"
	 */
	
	public function ComboEvent($combo_base, $combo_dest,$carac){
		$fs = explode(",",$carac);
		$fs = str_replace("'","",$fs);
		$store = $combo_dest."_store";
		$rec = $combo_dest ."_rec";
		$js = "
		alg_Form.findField('$combo_base').on('select',function(a,b){			
			var value = a.getValue().split(' ');
			if(value[1] == 'varchar(255)'){
				alg_Form.findField('$combo_dest').clearValue();
				$store.removeAll();
				$store.add(new $rec( { ". $fs[0] .": 'double', ". $fs[1] . ": 'double'}));
				$store.add(new $rec( { ". $fs[0] .": 'date', ". $fs[1] . ": 'date'}));
				$store.add(new $rec( { ". $fs[0] .": 'datetime', ". $fs[1] . ": 'datetime'}));
				$store.add(new $rec( { ". $fs[0] .": 'time', ". $fs[1] . ": 'time'}));
			}else if(value[1] == 'double'){
				alg_Form.findField('$combo_dest').clearValue();
				$store.removeAll();
				$store.add(new $rec( { ". $fs[0] .": 'varchar(255)', ". $fs[1] . ": 'varchar(255)'}));
			}else if(value[1] == 'date'){
				$store.removeAll();
				$store.add(new $rec( { ". $fs[0] .": 'varchar(255)', ". $fs[1] . ": 'varchar(255)'}));				
			}else if(value[1] == 'datetime'){
				$store.removeAll();
				$store.add(new $rec( { ". $fs[0] .": 'varchar(255)', ". $fs[1] . ": 'varchar(255)'}));				
			}else if(value[1] == 'time'){
				$store.removeAll();
				$store.add(new $rec( { ". $fs[0] .": 'varchar(255)', ". $fs[1] . ": 'varchar(255)'}));				
			}
		});";
 		return $js;	
	}
	/**
	 * Create a new radio button for the form
	 * 
	 *
	 *@param string contains the Id of the text field it must be unique
	 *@param string the label for the text field
	 *@param string the regular expression to evaluate in this text field (posix) ex. "/^\d+$/"
	 *@param string the text to show if regular expression evaluation fails
	 *@param string true/false to allow blank
	 *@param string evaluate the value of the text field with the given string
	 *@param string the text to show if the value of the text field not match with the value given by $valueIn
	 *@param string used for input files...
	 */
	public function getUploadForm($label){
		$this->param_desc .= "FileUpload Textfield: String<br>";
		$this->upload_form ="		
		upForm = new Ext.form.Form({
				labelWidth: 20,
				labelAlign: 'right',
				name: 'uploadForm',
				fileUpload : true,
				method: 'POST',
				url:'../Carga/writeFile.php',
				baseParams: { action : 'upload' }
		});
		upForm.add(
			new Ext.form.TextField({
				fieldLabel: '$label',
				name: 'fileUpload',
				id: 'fileUpload',
				allowBlank: true,
				inputType: 'file',				
				invalidText: 'Invalid file extension',
				validator: function(value){
					if(value.search('.csv') == -1 && value.search('.xls') == -1 && value.search('.arff') == -1){
						return false;
					}else{
						return true;
					}	
				}
			})			
		);
		upForm.addButton({
			text: 'Upload',						
			handler: function(){
				submitForm();
			}			
		});
		upForm.render('disc_div');
		
		Ext.get('fileUpload').on('change',function(){
			Ext.get('fileUpload').validate();
		});
		function submitForm(){
			if (upForm.isValid()) {	
				Ext.MessageBox.wait('uploading file', 'Please wait');
				upForm.submit({ 
					success:function(form, action){						
						checkMessage(action.result.msg.files);
						Ext.MessageBox.hide();						
					},								
					failure: function(form, action) {
						Ext.MessageBox.hide();
					}
				});
			}	
		}

		function checkMessage(data){			
			if(data[0].fileStatus == 0){
				xajax_cargar(data[0].fileName,0);
			}else{
				xajax_open(data[0].fileStatus, data[0].fileName)
			}		
		}
		";
		return $this->upload_form;
	}			
	/**
	 * Return js string with all the boxes needed and the render procedure for the form
	 * 
	 *
	 *@param string contains the function to execute if the user press apply button
	 * 
	 *@return js string, the function to execute always return one parameter more, the values of the form
	 *	 
	 */
	public function getForm($execFunc){
		$this->dlg_form = "
		alg_Form = new Ext.form.Form({
		labelWidth: 100
		});
		";
		$this->dlg_form .= $this->boxes;
		
		$execFunc = substr($execFunc,0,strlen($execFunc)-1);
		$execFunc .= ",alg_Form.getValues(true));";

		$this->dlg_form .= "apply = alg_Form.addButton({
			text: 'Apply',
			id: 'apply',
			disabled: true,
			handler: function(){
				if(alg_Form.isValid()){
					dialog.destroy();
					Ext.MessageBox.wait('Processing Data', 'Please wait');
					$execFunc
				}
			}
		});
		alg_Form.addButton({
			text: 'Cancel',
			handler: function(){
				dialog.hide();
				dialog.destroy();
			}
		});
		alg_Form.render('disc_div');		
		if(SBItemStat == 0){
			apply.enable();
		}
		";
		$this->boxes = '';
		return $this->dlg_form;
	}
	
public function getFormRule($execFunc,$id){
		$this->dlg_form = "
		alg_Form = new Ext.form.Form({
		labelWidth: 100
		});
		";
		$this->dlg_form .= $this->boxes;
		
		$execFunc = substr($execFunc,0,strlen($execFunc)-1);
		$execFunc .= ",alg_Form.getValues(true));";

		$this->dlg_form .= "apply = alg_Form.addButton({
			text: 'Apply',
			id: 'apply',
			disabled: true,
			handler: function(){
				if(alg_Form.isValid()){					
					Ext.MessageBox.wait('Processing Data', 'Please wait');
					$execFunc
				}
			}
		});
		alg_Form.addButton({
			text: 'Cancel',
			handler: function(){
				dialog.hide();
				dialog.destroy();
			}
		});
		alg_Form.render('disc_div');		
		alg_Form.items.get('$id').on('focus',function(){document.getElementById('msgDiv').innerHTML = '&nbsp;';});
		if(SBItemStat == 0){
			apply.enable();
		}
		";
		$this->boxes = '';
		return $this->dlg_form;
	}	
	
	/**
	 * Return js string with the dialog box
	 * 
	 *@return js string, the dialog box
	 *	 
	 */
	
	public function getJS(){
		return $this->js_dlg;			
	}
	/**
	 * Return html string with the html structure to create the dialog and the form
	 * 
	 *
	 *@param string contains the title for the dialog
	 * 
	 *@return html string, use with xajax --> $xajax->assign('target','innerHTML',this)
	 *	 
	 */
	
	public function getFormStruct($title,$msg){
		$this->alg_title = $title;
		ob_start();
		?>	
		<div class="x-dlg-hd" ><?php echo $title;?></div>	
		<div class="x-dlg-bd">
			<div id="cont_div">
				<div class="x-dlg-tab">
					<div class="inner-tab">
					<div class="x-AlgorithmForm"><?php echo $msg; ?></div>
					<div>
						<div class="x-box-tl"><div class="x-box-tr"><div class="x-box-tc"></div></div></div>
							<div class="x-box-ml"><div class="x-box-mr"><div class="x-box-mc">      
								<div id="disc_div">
					
								</div>
								<div id="msgDiv" style="color:red;">&nbsp;
								</div>						
							</div></div></div>
							<div class="x-box-bl"><div class="x-box-br"><div class="x-box-bc"></div></div></div>
						</div>
					</div>
				</div>
			</div>
			<div id="help_div">
			
			</div>
		</div>
		<?php
		return ob_get_clean();
	}
	
	public function getHtmlHelp($Description, $Example){
		$HTMLhelp = $this->setHtmlHelp($Description,$Example);
		return  $HTMLhelp;		
	}
	
	private function setHtmlHelp($Description, $Example){
		$xy = explode(",",$this->contentSize);
		$h_c_width = $xy[0] - 10;
		$h_c_height = $xy[1] - 10;
		ob_start();
//#CCCC99		
		?>
		<table width="<?php echo $h_c_width; ?>px;" height="<?php echo $h_c_height; ?>px" bgcolor="#E6E6CC">
		<tr>
			<td class="help-heads">
				Description
			</td>
		</tr>
		<tr>
			<td class="help-content">
				<?php echo $this->alg_title . " : " . $Description;  ?>
			</td>
		</tr>
		<tr>
			<td class="help-heads">
				Parameters
			</td>
		</tr>
		<tr>
			<td class="help-content">
				<?php echo $this->param_desc; ?>
			</td>
		</tr>
		<tr>
			<td class="help-heads">
				Example
			</td>
		</tr>
		<tr>
			<td class="help-content">
				<?php echo $Example;  ?>
			</td>
		</tr>
		</table>		
		
		<?php
		return ob_get_clean();
	}
}
?>