//
// Ext.ux.Multiselect
// 
// ---------------------------------------------------------------------------------------
// Version History
// ---------------------------------------------------------------------------------------
//
// 1.2     Enhancements                              23 Aug  2007          Figtree Systems
//          - Added ItemSelector (beta)
// 
// 1.1     Fixes/Enhancements                        16 Aug  2007          Figtree Systems
//          - Can now bind to external data store
//          - List items are now unselectable (prevents text highlight/selection)
//          - Enabled/disabled now works correctly
//          - Reset now properly clears selections
//          - Click and Change events added 
// 
// 1.0     Release                                   14 Aug  2007          Figtree Systems
//
//

Ext.namespace("Ext.ux");

Ext.ux.Multiselect = function(config){
	Ext.ux.Multiselect.superclass.constructor.call(this, config);

	this.addEvents({
		'dblclick' : true,
		'click' : true,
		'change' : true
	});

};

Ext.extend(Ext.ux.Multiselect, Ext.form.Field,  {

	name: '',
	defaultAutoCreate : {tag: "input", type: "hidden", value: ""},
	dataFields : [],
	data : [],
	width : 100,
	height : 100,
	store: null,
	displayField: 0,
	valueField: 1,
	dragable: false,
	view: null,

	onRender : function(ct, position){

		this.el = ct.createChild(); 
		this.el.dom.style.zoom = 1;

		var div = this.el.createChild({tag: div});

		if (!this.store) {
			this.store = new Ext.data.SimpleStore({
				fields: this.dataFields,
				data : this.data
			});
		}
			
		var cls = 'x-combo-list';
		
		this.list = new Ext.Layer({
			shadow: false, cls: [cls, 'ux-mselect'].join(' '), constrain:false
		}, div);
		
		var lw = this.width;
		var lh = this.height;
		
		this.list.setWidth(lw);
		this.list.setHeight(lh);
		this.list.swallowEvent('mousewheel');
		
		this.innerList = this.list.createChild({tag: 'div', cls: cls + '-inner'});
		this.innerList.setWidth(lw - this.list.getFrameWidth('lr'));
		this.innerList.setHeight(lh - this.list.getFrameWidth('tb'));
		
		if (Ext.isIE || Ext.isIE7)
		{
			var tpl = '<div unselectable=on class="' + cls + '-item ux-mselect-pointer">{' + this.displayField + '}</div>';
		}
		else
		{
			var tpl = '<div class="' + cls + '-item ux-mselect-pointer x-unselectable">{' + this.displayField + '}</div>';
		}
		
		this.view = new Ext.ux.DragableView(this.innerList, tpl, {
		multiSelect: true, store: this.store, selectedClass: 'x-combo-selected', dragable: this.dragable
		});
		
		this.view.on('click', this.onViewClick, this);
		this.view.on('beforeClick', this.onViewBeforeClick, this);
		this.view.on('dblclick', this.onViewDblClick, this);
		
		this.list.setStyle('position', '');
		this.list.show();
		
		this.defaultAutoCreate.name = this.name;
		this.hiddenField = this.el.createChild(this.defaultAutoCreate);
	},

	onViewClick: function(vw, index, node, e) {
		var arrayIndex = this.preClickSelections.indexOf(index);
		if (arrayIndex  != -1)
		{
			this.preClickSelections.splice(arrayIndex, 1);
			this.view.clearSelections(true);
			this.view.select(this.preClickSelections);
		}
		this.fireEvent('change', this, this.getValue(), this.hiddenField.dom.value);
		this.hiddenField.dom.value = this.getValue();
		this.fireEvent('click', this, e);
		
	},

	onViewBeforeClick: function(vw, index, node, e) {
		this.preClickSelections = this.view.getSelectedIndexes();
		if (this.disabled) {return false;}
	},

	onViewDblClick : function(vw, index, node, e) {
		this.fireEvent('dblclick', vw, index);
	},

	getValue: function(valueField){
		var returnArray = [];
		var selectionsArray = this.view.getSelectedIndexes();
		if (selectionsArray.length == 0) {return '';}
		for (var i=0; i<selectionsArray.length; i++)
		{
			returnArray.push(this.store.getAt(selectionsArray[i]).get(((valueField != null)? valueField : this.valueField)));
		}
		return returnArray.join(',');
	},

	setValue: function(values)
	{
		var index;
		var selections = [];
		this.view.clearSelections();
		this.hiddenField.dom.value = '';
		
		if (!values || (values == ''))
		{
			return;
		}
		
		if (!(values instanceof Array))
		{
			values = values.split(',');
		}
		for (var i=0; i<values.length; i++)
		{
			index = this.view.store.indexOf(this.view.store.query(this.valueField, values[i]).itemAt(0));
			selections.push(index);
		}
		this.view.select(selections);
		this.hiddenField.dom.value = this.getValue();
	},

	getListValues: function(valueField){
		var returnArray = [];
		for (var i=0; i<this.view.store.getCount(); i++)
		{
			returnArray.push(this.store.getAt(i).get(((valueField != null)? valueField : this.valueField)));
		}
		return returnArray.join(',');
	},

	getSelectedRecords: function() {
		return this.view.getSelectedRecords();
	},

	takeSelectedRecords: function() {
		return this.view.takeSelectedRecords();
	},

	addRecords: function(recordArray){
		this.view.addRecords(recordArray);
	},
	/*-*/
	deleteAll: function(recordArray){
		this.view.deleteAll(recordArray);
	},
	/*-*/
	reset : function() {
		this.setValue('');
	}
});

// Dragable view for use in Ext.ux.ItemSelector (future)
Ext.ux.DragableView = function(id, tpl, config){
	Ext.ux.DragableView.superclass.constructor.call(this, id, tpl, config);

	this.group = this.el.dom.id
	var myDropTarget = new Ext.dd.DropTarget(this.el);
	myDropTarget.addToGroup(this.group);
	
	this.addEvents('storeUpdated', true);
	this.store.on('add', this.storeChanged, this);
	this.store.on('remove', this.storeChanged, this);
};

Ext.extend(Ext.ux.DragableView, Ext.View,  {

	dropTarget : null,

	dragable : false,

	refresh : function() {
		/**/
		this.dragable = false;
		/**/
		Ext.ux.DragableView.superclass.refresh.call(this);
		if (this.dragable)
		{
			for (i=0; i<this.nodes.length; i++)
			{
				new Ext.ux.DragableViewProxy(this.nodes[i], ((this.dropTarget != null) ? this.dropTarget.el.dom.id : ''), this);
			}
		}
	},

	getSelectedRecords: function() {
		var returnArray = [];
		var selectionsArray = this.getSelectedIndexes();
		for (var i=0; i<selectionsArray.length; i++)
		{
			returnArray.push(this.store.getAt(selectionsArray[i]));
		}
		return returnArray;
	},

	takeSelectedRecords: function() {
		var returnArray = this.getSelectedRecords();
		for(var i=0; i<returnArray.length; i++)
		{
			this.store.remove(returnArray[i]);
		}
		this.refresh();
		return returnArray;
	},

	addRecords: function(recordArray){
		this.store.add(recordArray);
		this.refresh();
	},
	/**/
	deleteAll: function(recordArray){
		var storeArray =  this.store.getRange();
		for(var i = 0; i<storeArray.length; i++){
			this.store.remove(storeArray[i]);			
		}
		this.refresh();
	},
	/**/
	storeChanged : function() {
		this.fireEvent('storeUpdated', this);		
	}

});

Ext.ux.DragableViewProxy = function(node, group, parentView, config){
	Ext.ux.DragableViewProxy.superclass.constructor.call(this, node, group, config);
	this.parentView = parentView;
	this.viewIndex = this.parentView.indexOf(this.parentView.findItemFromChild(node));
	this.record = this.parentView.store.getAt(this.viewIndex);
};

Ext.extend(Ext.ux.DragableViewProxy, Ext.dd.DDProxy, {
	startDrag: function(x, y) {
		// make the proxy look like the source element
		var dragEl = Ext.get(this.getDragEl());
		var clickEl = Ext.get(this.getEl());

		var padding = clickEl.getPadding('t') + 'px '
		  + clickEl.getPadding('r') + 'px '
	 	  + clickEl.getPadding('b') + 'px '
		  + clickEl.getPadding('l') + 'px';

		dragEl.dom.innerHTML = '<div style="padding:' + padding + ';"></div>';

		dragEl.setStyle(clickEl.getStyles('background-color', 'color', 'padding'));
		dragEl.setStyle('border', '1px solid gray');
		dragEl.setStyle('z-index', '90000');
    	},

	onDrag: function(e) {
		this.droppedOn = '';
	},

	onDragDrop: function(e, id) {
		this.droppedOn = id;
	},

	endDrag: function(e) {
		if (this.parentView.dropTarget != null && this.droppedOn == this.parentView.dropTarget.el.dom.id)
		{
			if (!this.parentView.isSelected(this.viewIndex))
			{
				this.parentView.store.remove(this.record);
				this.parentView.dropTarget.store.add(this.record);
			}
			
			var selectedIndexes = this.parentView.getSelectedIndexes();
			if (selectedIndexes.length > 0)
			{
				this.parentView.dropTarget.store.add(this.parentView.takeSelectedRecords());
				this.parentView.dropTarget.refresh();
			}
			else
			{
				this.parentView.refresh();
				this.parentView.dropTarget.refresh();
			}
		}
		else if(this.droppedOn != '')
		{
			/* Attempt at being able to reposition (up/down) multiple selected items
			 * ... but it doesnt work :(
			 * 
			var node = Ext.getDom(this.droppedOn);
			var view = this.parentView;
			//var records = view.getSelectedRecords();
			var index = view.indexOf(view.findItemFromChild(node));
			var records = [];
			var selectionsArray = view.getSelectedIndexes();

			selectionsArray.sort();
			selectionsArray.reverse();
			for (var i=0; i<selectionsArray.length; i++) {
				record = view.store.getAt(selectionsArray[i]);
				records.push(record);
			}

			for(var i=0; i<records.length; i++) {
				view.store.remove(records[i]);
			}
			
			for (var i=0; i<records.length; i++) {
				record = records[i];
				view.store.insert(index, record);
			}

			view.refresh();*/
			
			var node = Ext.getDom(this.droppedOn);
			var index = this.parentView.indexOf(this.parentView.findItemFromChild(node));
			this.parentView.store.remove(this.record);
			this.parentView.store.insert(index, this.record);
			this.parentView.refresh();
			
		}
		else
		{
			e.cancel = true;
		}
	}
});

Ext.ux.ItemSelector = function(config){
	Ext.ux.ItemSelector.superclass.constructor.call(this, config);
};

Ext.extend(Ext.ux.ItemSelector, Ext.form.Field,  {

	defaultAutoCreate : {tag: "input", type: "hidden", value: ""},
	dataFields : [],
	fromData : [],
	toData : [],
	/**/
	fromLabel : "",
	toLabel : "",
	/**/
	fromStore : null,
	toStore : null,
	msWidth : 100,
	msHeight : 100,
	store: null,
	displayField: 0,
	valueField: 1,
	
	onRender : function(ct, position){

		this.iconUp = (this.iconUp) ? this.iconUp : "../../icons/up.gif";
		this.iconDown = (this.iconDown) ? this.iconDown : "../../icons/down.gif";
		this.iconLeft = (this.iconLeft) ? this.iconLeft : "../../icons/left.gif";
		this.iconRight = (this.iconRight) ? this.iconRight : "../../icons/right.gif";
		this.iconALeft = (this.iconALeft) ? this.iconALeft : "../../icons/Aleft.gif";
		this.iconARight = (this.iconARight) ? this.iconARight : "../../icons/Aright.gif";
		
		var table = ct.createChild({tag: 'table', cellpadding: 0, cellspacing: 0});
		this.el = table;
		var tr = table.createChild({tag: 'tr'});
		tr = tr.child('tr'); // Line above auto creates <tbody> tag???
		var td = Ext.get(tr.dom.appendChild(document.createElement('td')));
		var divFrom = td.createChild({tag: 'div',html:this.fromLabel, style: 'font-family:Arial, Helvetica, sans-serif;font-size:12px;color:#000000;font-weight:bolder;'});

		this.fromMultiselect = new Ext.ux.Multiselect({dragable: true, width: this.msWidth, height: this.msHeight, dataFields: this.dataFields, data: this.fromData, noHiddenField: true, displayField: this.displayField, valueField: this.valueField, store: this.fromStore});
		this.fromMultiselect.on('dblclick', this.fromDblClick, this);

		td = Ext.get(tr.dom.appendChild(document.createElement('td')));
		td.dom.align = 'center';
		td.dom.style.paddingLeft = '5px';
		td.dom.style.paddingRight = '5px';
		
	//	if(CleanMatch == 'clean' || CleanMatch == 'match'){
			this.upIcon = td.createChild({tag:'img', src:this.iconUp, style:{cursor:'pointer', margin:'2px'}});
			this.upIcon.on('click', this.up, this);
			td.createChild({tag: 'br'});
	//	}
		/*new code to add the button to addall items*/
		this.addAllIcon = td.createChild({tag:'img', src:this.iconARight, style:{cursor:'pointer', margin:'2px'}});
		this.addAllIcon.on('click', this.allFromTo, this);
		td.createChild({tag: 'br'});		
		/**/
		this.addIcon = td.createChild({tag:'img', src:this.iconRight, style:{cursor:'pointer', margin:'2px'}});
		this.addIcon.on('click', this.fromTo, this);
		td.createChild({tag: 'br'});
		this.removeIcon = td.createChild({tag:'img', src:this.iconLeft, style:{cursor:'pointer', margin:'2px'}});
		this.removeIcon.on('click', this.toFrom, this);
		td.createChild({tag: 'br'});
		/*new code to add the button to removeall items*/
		this.removeAllIcon = td.createChild({tag:'img', src:this.iconALeft, style:{cursor:'pointer', margin:'2px'}});
		this.removeAllIcon.on('click', this.allToFrom, this);
		td.createChild({tag: 'br'});		
		/**/
//		if(CleanMatch == 'clean' || CleanMatch == 'match'){
			this.downIcon = td.createChild({tag:'img', src:this.iconDown, style:{cursor:'pointer', margin:'2px'}});
			this.downIcon.on('click', this.down, this);
//		}
		td = Ext.get(tr.dom.appendChild(document.createElement('td')));

		divTo = td.createChild({tag: 'div',html:this.toLabel,style: 'font-family:Arial, Helvetica, sans-serif;font-size:12px;color:#000000;font-weight:bolder;'});

		this.toMultiselect = new Ext.ux.Multiselect({dragable: true, width: this.msWidth, height: this.msHeight, dataFields: this.dataFields, data: this.toData, noHiddenField: true, displayField: this.displayField, valueField: this.valueField, store: this.toStore});
		this.toMultiselect.on('dblclick', this.toDblClick, this);

		this.fromMultiselect.applyTo(divFrom);
		this.toMultiselect.applyTo(divTo);

		this.fromMultiselect.view.dropTarget = this.toMultiselect.view;
		this.toMultiselect.view.dropTarget = this.fromMultiselect.view;

		this.fromMultiselect.view.refresh();
		this.toMultiselect.view.refresh();

		this.defaultAutoCreate.name = this.name;
		this.hiddenField = ct.createChild(this.defaultAutoCreate);
		
		this.toMultiselect.view.on('storeUpdated', this.valueChanged, this);
		
		this.valueChanged();
		
	},

	fromTo : function(e)
	{
		this.toMultiselect.addRecords(this.fromMultiselect.takeSelectedRecords());
		/**/
		apply.disable();
		if((this.toMultiselect.view.store.getCount() > 0 && CleanMatch == 'clean') || (this.toMultiselect.view.store.getCount() == 2 && CleanMatch == 'match')){
			apply.enable();
		}
		if(CleanMatch == 'match2'){
			matchFiltersEnable = false;
		}
		/**/
	},

	toFrom : function(e)
	{
		
		this.fromMultiselect.addRecords(this.toMultiselect.takeSelectedRecords());
		/**/
		apply.enable();	
		if((this.toMultiselect.view.store.getCount() == 0 && CleanMatch == 'clean') || (this.toMultiselect.view.store.getCount() != 2 && CleanMatch == 'match')){
			apply.disable();
		}
		if(CleanMatch == 'match2'){
			matchFiltersEnable = false;
		}
		
		/**/
	},

	fromDblClick : function (view, index, e)
	{
		var record = this.fromMultiselect.view.store.getAt(index);
		this.fromMultiselect.view.store.remove(record);
		this.toMultiselect.addRecords(record);
		/**/
		apply.enable();
		if(CleanMatch == 'match' && this.toMultiselect.view.store.getCount() != 2){
			apply.disable();
		}
		if(CleanMatch == 'match2'){
			matchFiltersEnable = false;
		}
		
		/**/
	},

	toDblClick : function (view, index, e)
	{
		var record = this.toMultiselect.view.store.getAt(index);
		this.toMultiselect.view.store.remove(record);
		this.fromMultiselect.addRecords(record);
		/**/
		apply.enable();
		if((this.toMultiselect.view.store.getCount() == 0 && CleanMatch == 'clean') || (this.toMultiselect.view.store.getCount() != 2 && CleanMatch == 'match')){
			apply.disable();
		}
		if(CleanMatch == 'match2'){
			matchFiltersEnable = false;
		}

		/**/
	},
	/**/
	allFromTo : function(e)
	{
		var record = this.fromMultiselect.view.store.getRange();		
		this.toMultiselect.addRecords(record);
		this.fromMultiselect.deleteAll(record);
		apply.enable();		
		if(CleanMatch == 'match' && this.toMultiselect.view.store.getCount() > 2){
			apply.disable();
		}
		if(CleanMatch == 'match2'){
			matchFiltersEnable = false;
		}

		
	},
	allToFrom : function(e)
	{
		var record = this.toMultiselect.view.store.getRange();		
		this.fromMultiselect.addRecords(record);
		this.toMultiselect.deleteAll(record);
		apply.disable();
		if(CleanMatch == 'match2'){
			matchFiltersEnable = false;
		}
		
	},	
	/**/
	getValue : function()
	{
		return this.hiddenField.dom.value;
	},
	
	valueChanged : function()
	{
		this.hiddenField.dom.value = this.toMultiselect.getListValues();
	},
	
	enable : function(){
		this.fromMultiselect.enable();
		this.toMultiselect.enable();
		Ext.ux.ItemSelector.superclass.enable.call(this);
	},

	disable : function(){
		this.fromMultiselect.disable();
		this.toMultiselect.disable();
		Ext.ux.ItemSelector.superclass.disable.call(this);
	},
	
	up : function() {
		var record = null;
		var selectionsArray = this.toMultiselect.view.getSelectedIndexes();
		selectionsArray.sort();
		var newSelectionsArray = [];
		if (selectionsArray.length > 0) {
			for (var i=0; i<selectionsArray.length; i++) {
				record = this.toMultiselect.view.store.getAt(selectionsArray[i]);
				if ((selectionsArray[i] - 1) >= 0) {
					this.toMultiselect.view.store.remove(record);
					this.toMultiselect.view.store.insert(selectionsArray[i] - 1, record);
					newSelectionsArray.push(selectionsArray[i] - 1);
				}
			}
			this.toMultiselect.view.refresh();
			this.toMultiselect.view.select(newSelectionsArray);
		}
		if(CleanMatch == 'match2'){
			matchFiltersEnable = false;
		}
		
	},

	down : function() {
		var record = null;
		var selectionsArray = this.toMultiselect.view.getSelectedIndexes();
		selectionsArray.sort();
		selectionsArray.reverse();
		var newSelectionsArray = [];
		if (selectionsArray.length > 0) {
			for (var i=0; i<selectionsArray.length; i++) {
				record = this.toMultiselect.view.store.getAt(selectionsArray[i]);
				if ((selectionsArray[i] + 1) < this.toMultiselect.view.store.getCount()) {
					this.toMultiselect.view.store.remove(record);
					this.toMultiselect.view.store.insert(selectionsArray[i] + 1, record);
					newSelectionsArray.push(selectionsArray[i] + 1);
				}
			}
			this.toMultiselect.view.refresh();
			this.toMultiselect.view.select(newSelectionsArray);
		}
		if(CleanMatch == 'match2'){
			matchFiltersEnable = false;
		}
		
	}
});

